Method: Net::SSH::Connection::EventLoop#process

Defined in:
lib/net/ssh/connection/event_loop.rb

#process(wait = nil, &block) ⇒ Object

process until timeout if a block is given a session will be removed from loop if block returns false for that session



27
28
29
30
31
# File 'lib/net/ssh/connection/event_loop.rb', line 27

def process(wait = nil, &block)
  return false unless ev_preprocess(&block)

  ev_select_and_postprocess(wait)
end