Class: Net::SSH::Transport::Session
- Inherits:
-
Object
- Object
- Net::SSH::Transport::Session
- Defined in:
- lib/net/ssh/transport/session.rb
Overview
The transport layer represents the lowest level of the SSH protocol, and implements basic message exchanging and protocol initialization. It will never be instantiated directly (unless you really know what you’re about), but will instead be created for you automatically when you create a new SSH session via Net::SSH.start.
Constant Summary collapse
- DEFAULT_PORT =
The standard port for the SSH protocol.
22
Constants included from Constants
Constants::DEBUG, Constants::DISCONNECT, Constants::IGNORE, Constants::KEXDH_INIT, Constants::KEXDH_REPLY, Constants::KEXECDH_INIT, Constants::KEXECDH_REPLY, Constants::KEXINIT, Constants::NEWKEYS, Constants::SERVICE_ACCEPT, Constants::SERVICE_REQUEST, Constants::UNIMPLEMENTED
Instance Attribute Summary collapse
-
#algorithms ⇒ Object
readonly
The Algorithms instance used to perform key exchanges.
-
#host ⇒ Object
readonly
The host to connect to, as given to the constructor.
-
#host_key_verifier ⇒ Object
readonly
The host-key verifier object used to verify host keys, to ensure that the connection is not being spoofed.
-
#options ⇒ Object
readonly
The hash of options that were given to the object at initialization.
-
#port ⇒ Object
readonly
The port number to connect to, as given in the options to the constructor.
-
#queue ⇒ Object
readonly
this method is primarily for use in tests.
-
#server_version ⇒ Object
readonly
The ServerVersion instance that encapsulates the negotiated protocol version.
-
#socket ⇒ Object
readonly
The underlying socket object being used to communicate with the remote host.
Attributes included from Loggable
Instance Method Summary collapse
-
#close ⇒ Object
Cleans up (see PacketStream#cleanup) and closes the underlying socket.
-
#closed? ⇒ Boolean
Returns true if the underlying socket has been closed.
-
#configure_client(options = {}) ⇒ Object
Configure’s the packet stream’s client state with the given set of options.
-
#configure_server(options = {}) ⇒ Object
Configure’s the packet stream’s server state with the given set of options.
-
#enqueue_message(message) ⇒ Object
Enqueues the given message, such that it will be sent at the earliest opportunity.
-
#hint(which, value = true) ⇒ Object
Sets a new hint for the packet stream, which the packet stream may use to change its behavior.
-
#host_as_string ⇒ Object
Returns the host (and possibly IP address) in a format compatible with SSH known-host files.
-
#initialize(host, options = {}) ⇒ Session
constructor
Instantiates a new transport layer abstraction.
-
#next_message ⇒ Object
Blocks until a new packet is available to be read, and returns that packet.
-
#peer ⇒ Object
Returns a hash of information about the peer (remote) side of the socket, including :ip, :port, :host, and :canonized (see #host_as_string).
-
#poll_message(mode = :nonblock, consume_queue = true) ⇒ Object
Tries to read the next packet from the socket.
-
#push(packet) ⇒ Object
Adds the given packet to the packet queue.
-
#rekey! ⇒ Object
Requests a rekey operation, and blocks until the operation completes.
-
#rekey_as_needed ⇒ Object
Returns immediately if a rekey is already in process.
-
#send_message(message) ⇒ Object
Sends the given message via the packet stream, blocking until the entire message has been sent.
-
#service_request(service) ⇒ Object
Returns a new service_request packet for the given service name, ready for sending to the server.
-
#shutdown! ⇒ Object
Performs a “hard” shutdown of the connection.
-
#wait ⇒ Object
Waits (blocks) until the given block returns true.
Methods included from Loggable
#debug, #error, #fatal, #info, #lwarn
Constructor Details
#initialize(host, options = {}) ⇒ Session
Instantiates a new transport layer abstraction. This will block until the initial key exchange completes, leaving you with a ready-to-use transport session.
57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 |
# File 'lib/net/ssh/transport/session.rb', line 57 def initialize(host, ={}) self.logger = [:logger] @host = host @port = [:port] || DEFAULT_PORT @bind_address = [:bind_address] || nil = debug { "establishing connection to #{@host}:#{@port}" } factory = [:proxy] || TCPSocket @socket = timeout([:timeout] || 0) { case when [:proxy] then factory.open(@host, @port, ) when @bind_address.nil? then factory.open(@host, @port) else factory.open(@host, @port, @bind_address) end } @socket.extend(PacketStream) @socket.logger = @logger debug { "connection established" } @queue = [] @host_key_verifier = select_host_key_verifier([:paranoid]) @server_version = timeout([:timeout] || 0) { ServerVersion.new(socket, logger) } @algorithms = Algorithms.new(self, ) wait { algorithms.initialized? } end |
Instance Attribute Details
#algorithms ⇒ Object (readonly)
The Algorithms instance used to perform key exchanges.
45 46 47 |
# File 'lib/net/ssh/transport/session.rb', line 45 def algorithms @algorithms end |
#host ⇒ Object (readonly)
The host to connect to, as given to the constructor.
30 31 32 |
# File 'lib/net/ssh/transport/session.rb', line 30 def host @host end |
#host_key_verifier ⇒ Object (readonly)
The host-key verifier object used to verify host keys, to ensure that the connection is not being spoofed.
49 50 51 |
# File 'lib/net/ssh/transport/session.rb', line 49 def host_key_verifier @host_key_verifier end |
#options ⇒ Object (readonly)
The hash of options that were given to the object at initialization.
52 53 54 |
# File 'lib/net/ssh/transport/session.rb', line 52 def end |
#port ⇒ Object (readonly)
The port number to connect to, as given in the options to the constructor. If no port number was given, this will default to DEFAULT_PORT.
34 35 36 |
# File 'lib/net/ssh/transport/session.rb', line 34 def port @port end |
#queue ⇒ Object (readonly)
this method is primarily for use in tests
258 259 260 |
# File 'lib/net/ssh/transport/session.rb', line 258 def queue @queue end |
#server_version ⇒ Object (readonly)
The ServerVersion instance that encapsulates the negotiated protocol version.
42 43 44 |
# File 'lib/net/ssh/transport/session.rb', line 42 def server_version @server_version end |
#socket ⇒ Object (readonly)
The underlying socket object being used to communicate with the remote host.
38 39 40 |
# File 'lib/net/ssh/transport/session.rb', line 38 def socket @socket end |
Instance Method Details
#close ⇒ Object
Cleans up (see PacketStream#cleanup) and closes the underlying socket.
111 112 113 114 |
# File 'lib/net/ssh/transport/session.rb', line 111 def close socket.cleanup socket.close end |
#closed? ⇒ Boolean
Returns true if the underlying socket has been closed.
106 107 108 |
# File 'lib/net/ssh/transport/session.rb', line 106 def closed? socket.closed? end |
#configure_client(options = {}) ⇒ Object
Configure’s the packet stream’s client state with the given set of options. This is typically used to define the cipher, compression, and hmac algorithms to use when sending packets to the server.
238 239 240 |
# File 'lib/net/ssh/transport/session.rb', line 238 def configure_client(={}) socket.client.set() end |
#configure_server(options = {}) ⇒ Object
Configure’s the packet stream’s server state with the given set of options. This is typically used to define the cipher, compression, and hmac algorithms to use when reading packets from the server.
245 246 247 |
# File 'lib/net/ssh/transport/session.rb', line 245 def configure_server(={}) socket.server.set() end |
#enqueue_message(message) ⇒ Object
Enqueues the given message, such that it will be sent at the earliest opportunity. This does not block, but returns immediately.
231 232 233 |
# File 'lib/net/ssh/transport/session.rb', line 231 def () socket.enqueue_packet() end |
#hint(which, value = true) ⇒ Object
Sets a new hint for the packet stream, which the packet stream may use to change its behavior. (See PacketStream#hints).
251 252 253 |
# File 'lib/net/ssh/transport/session.rb', line 251 def hint(which, value=true) socket.hints[which] = value end |
#host_as_string ⇒ Object
Returns the host (and possibly IP address) in a format compatible with SSH known-host files.
92 93 94 95 96 97 98 99 100 101 102 103 |
# File 'lib/net/ssh/transport/session.rb', line 92 def host_as_string @host_as_string ||= begin string = "#{host}" string = "[#{string}]:#{port}" if port != DEFAULT_PORT if socket.peer_ip != host string2 = socket.peer_ip string2 = "[#{string2}]:#{port}" if port != DEFAULT_PORT string << "," << string2 end string end end |
#next_message ⇒ Object
Blocks until a new packet is available to be read, and returns that packet. See #poll_message.
157 158 159 |
# File 'lib/net/ssh/transport/session.rb', line 157 def (:block) end |
#peer ⇒ Object
Returns a hash of information about the peer (remote) side of the socket, including :ip, :port, :host, and :canonized (see #host_as_string).
151 152 153 |
# File 'lib/net/ssh/transport/session.rb', line 151 def peer @peer ||= { :ip => socket.peer_ip, :port => @port.to_i, :host => @host, :canonized => host_as_string } end |
#poll_message(mode = :nonblock, consume_queue = true) ⇒ Object
Tries to read the next packet from the socket. If mode is :nonblock (the default), this will not block and will return nil if there are no packets waiting to be read. Otherwise, this will block until a packet is available. Note that some packet types (DISCONNECT, IGNORE, UNIMPLEMENTED, DEBUG, and KEXINIT) are handled silently by this method, and will never be returned.
If a key-exchange is in process and a disallowed packet type is received, it will be enqueued and otherwise ignored. When a key-exchange is not in process, and consume_queue is true, packets will be first read from the queue before the socket is queried.
172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 |
# File 'lib/net/ssh/transport/session.rb', line 172 def (mode=:nonblock, consume_queue=true) loop do if consume_queue && @queue.any? && algorithms.allow?(@queue.first) return @queue.shift end packet = socket.next_packet(mode) return nil if packet.nil? case packet.type when DISCONNECT raise Net::SSH::Disconnect, "disconnected: #{packet[:description]} (#{packet[:reason_code]})" when IGNORE debug { "IGNORE packet recieved: #{packet[:data].inspect}" } when UNIMPLEMENTED lwarn { "UNIMPLEMENTED: #{packet[:number]}" } when DEBUG send(packet[:always_display] ? :fatal : :debug) { packet[:message] } when KEXINIT algorithms.accept_kexinit(packet) else return packet if algorithms.allow?(packet) push(packet) end end end |
#push(packet) ⇒ Object
Adds the given packet to the packet queue. If the queue is non-empty, #poll_message will return packets from the queue in the order they were received.
219 220 221 |
# File 'lib/net/ssh/transport/session.rb', line 219 def push(packet) @queue.push(packet) end |
#rekey! ⇒ Object
Requests a rekey operation, and blocks until the operation completes. If a rekey is already pending, this returns immediately, having no effect.
134 135 136 137 138 139 |
# File 'lib/net/ssh/transport/session.rb', line 134 def rekey! if !algorithms.pending? algorithms.rekey! wait { algorithms.initialized? } end end |
#rekey_as_needed ⇒ Object
Returns immediately if a rekey is already in process. Otherwise, if a rekey is needed (as indicated by the socket, see PacketStream#if_needs_rekey?) one is performed, causing this method to block until it completes.
144 145 146 147 |
# File 'lib/net/ssh/transport/session.rb', line 144 def rekey_as_needed return if algorithms.pending? socket.if_needs_rekey? { rekey! } end |
#send_message(message) ⇒ Object
Sends the given message via the packet stream, blocking until the entire message has been sent.
225 226 227 |
# File 'lib/net/ssh/transport/session.rb', line 225 def () socket.send_packet() end |
#service_request(service) ⇒ Object
Returns a new service_request packet for the given service name, ready for sending to the server.
127 128 129 |
# File 'lib/net/ssh/transport/session.rb', line 127 def service_request(service) Net::SSH::Buffer.from(:byte, SERVICE_REQUEST, :string, service) end |
#shutdown! ⇒ Object
Performs a “hard” shutdown of the connection. In general, this should never be done, but it might be necessary (in a rescue clause, for instance, when the connection needs to close but you don’t know the status of the underlying protocol’s state).
120 121 122 123 |
# File 'lib/net/ssh/transport/session.rb', line 120 def shutdown! error { "forcing connection closed" } socket.close end |
#wait ⇒ Object
Waits (blocks) until the given block returns true. If no block is given, this just waits long enough to see if there are any pending packets. Any packets read are enqueued (see #push).
207 208 209 210 211 212 213 214 |
# File 'lib/net/ssh/transport/session.rb', line 207 def wait loop do break if block_given? && yield = (:nonblock, false) push() if break if !block_given? end end |