Class: OpenSSL::PKey::EC::Point
- Inherits:
-
Object
- Object
- OpenSSL::PKey::EC::Point
- Defined in:
- lib/net/ssh/transport/openssl.rb
Instance Method Summary collapse
-
#ssh_type ⇒ Object
Returns the description of this key type used by the SSH2 protocol, like “ecdsa-sha2-nistp256”.
-
#to_blob ⇒ Object
Converts the key to a blob, according to the SSH2 protocol.
Instance Method Details
#ssh_type ⇒ Object
Returns the description of this key type used by the SSH2 protocol, like “ecdsa-sha2-nistp256”
232 233 234 |
# File 'lib/net/ssh/transport/openssl.rb', line 232 def ssh_type "ecdsa-sha2-#{CurveNameAliasInv[self.group.curve_name]}" end |
#to_blob ⇒ Object
Converts the key to a blob, according to the SSH2 protocol.
237 238 239 240 241 242 |
# File 'lib/net/ssh/transport/openssl.rb', line 237 def to_blob @blob ||= Net::SSH::Buffer.from(:string, ssh_type, :string, CurveNameAliasInv[self.group.curve_name], :mstring, self.to_bn.to_s(2)).to_s @blob end |