Class: Net::SSH::KnownHosts
- Inherits:
-
Object
- Object
- Net::SSH::KnownHosts
- Defined in:
- lib/net/ssh/known_hosts.rb
Overview
Searches an OpenSSH-style known-host file for a given host, and returns all matching keys. This is used to implement host-key verification, as well as to determine what key a user prefers to use for a given host.
This is used internally by Net::SSH, and will never need to be used directly by consumers of the library.
Constant Summary collapse
- SUPPORTED_TYPE =
%w[ssh-rsa ssh-dss]
Instance Attribute Summary collapse
-
#source ⇒ Object
readonly
The host-key file name that this KnownHosts instance will use to search for keys.
Class Method Summary collapse
-
.add(host, key, options = {}) ⇒ Object
Looks in all user known host files (see KnownHosts.hostfiles) and tries to add an entry for the given host and key to the first file it is able to.
-
.hostfiles(options, which = :all) ⇒ Object
Looks in the given
optionshash for the :user_known_hosts_file and :global_known_hosts_file keys, and returns an array of all known hosts files. -
.search_for(host, options = {}) ⇒ Object
Searches all known host files (see KnownHosts.hostfiles) for all keys of the given host.
-
.search_in(files, host) ⇒ Object
Search for all known keys for the given host, in every file given in the
filesarray.
Instance Method Summary collapse
-
#add(host, key) ⇒ Object
Tries to append an entry to the current source file for the given host and key.
-
#initialize(source) ⇒ KnownHosts
constructor
Instantiate a new KnownHosts instance that will search the given known-hosts file.
-
#keys_for(host) ⇒ Object
Returns an array of all keys that are known to be associatd with the given host.
-
#known_host_hash?(hostlist, entries) ⇒ Boolean
Indicates whether one of the entries matches an hostname that has been stored as a HMAC-SHA1 hash in the known hosts.
Constructor Details
#initialize(source) ⇒ KnownHosts
Instantiate a new KnownHosts instance that will search the given known-hosts file. The path is expanded file File.expand_path.
105 106 107 |
# File 'lib/net/ssh/known_hosts.rb', line 105 def initialize(source) @source = File.(source) end |
Instance Attribute Details
#source ⇒ Object (readonly)
The host-key file name that this KnownHosts instance will use to search for keys.
101 102 103 |
# File 'lib/net/ssh/known_hosts.rb', line 101 def source @source end |
Class Method Details
.add(host, key, options = {}) ⇒ Object
Looks in all user known host files (see KnownHosts.hostfiles) and tries to add an entry for the given host and key to the first file it is able to.
87 88 89 90 91 92 93 94 95 96 |
# File 'lib/net/ssh/known_hosts.rb', line 87 def add(host, key, ={}) hostfiles(, :user).each do |file| begin KnownHosts.new(file).add(host, key) return rescue SystemCallError # try the next hostfile end end end |
.hostfiles(options, which = :all) ⇒ Object
Looks in the given options hash for the :user_known_hosts_file and :global_known_hosts_file keys, and returns an array of all known hosts files. If the :user_known_hosts_file key is not set, the default is returned (~/.ssh/known_hosts and ~/.ssh/known_hosts2). If :global_known_hosts_file is not set, the default is used (/etc/ssh/ssh_known_hosts and /etc/ssh/ssh_known_hosts2).
If you only want the user known host files, you can pass :user as the second option.
74 75 76 77 78 79 80 81 82 |
# File 'lib/net/ssh/known_hosts.rb', line 74 def hostfiles(, which=:all) files = [] files += Array([:user_known_hosts_file] || %w[~/.ssh/known_hosts ~/.ssh/known_hosts2]) if which == :all || which == :user files += Array([:global_known_hosts_file] || %w[/etc/ssh/ssh_known_hosts /etc/ssh/ssh_known_hosts2]) if which == :all || which == :global return files end |
.search_for(host, options = {}) ⇒ Object
Searches all known host files (see KnownHosts.hostfiles) for all keys of the given host. Returns an enumerable of keys found.
55 56 57 |
# File 'lib/net/ssh/known_hosts.rb', line 55 def search_for(host, ={}) HostKeys.new(search_in(hostfiles(), host), host, self, ) end |
.search_in(files, host) ⇒ Object
Search for all known keys for the given host, in every file given in the files array. Returns the list of keys.
61 62 63 |
# File 'lib/net/ssh/known_hosts.rb', line 61 def search_in(files, host) files.flat_map { |file| KnownHosts.new(file).keys_for(host) } end |
Instance Method Details
#add(host, key) ⇒ Object
Tries to append an entry to the current source file for the given host and key. If it is unable to (because the file is not writable, for instance), an exception will be raised.
173 174 175 176 177 178 |
# File 'lib/net/ssh/known_hosts.rb', line 173 def add(host, key) File.open(source, "a") do |file| blob = [Net::SSH::Buffer.from(:key, key).to_s].pack("m*").gsub(/\s/, "") file.puts "#{host} #{key.ssh_type} #{blob}" end end |
#keys_for(host) ⇒ Object
Returns an array of all keys that are known to be associatd with the given host. The host parameter is either the domain name or ip address of the host, or both (comma-separated). Additionally, if a non-standard port is being used, it may be specified by putting the host (or ip, or both) in square brackets, and appending the port outside the brackets after a colon. Possible formats for host, then, are;
"net.ssh.test"
"1.2.3.4"
"net.ssh.test,1.2.3.4"
"[net.ssh.test]:5555"
"[1,2,3,4]:5555"
"[net.ssh.test]:5555,[1.2.3.4]:5555
122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 |
# File 'lib/net/ssh/known_hosts.rb', line 122 def keys_for(host) keys = [] return keys unless File.readable?(source) entries = host.split(/,/) File.open(source) do |file| scanner = StringScanner.new("") file.each_line do |line| scanner.string = line scanner.skip(/\s*/) next if scanner.match?(/$|#/) hostlist = scanner.scan(/\S+/).split(/,/) found = entries.all? { |entry| hostlist.include?(entry) } || known_host_hash?(hostlist, entries) next unless found scanner.skip(/\s*/) type = scanner.scan(/\S+/) next unless SUPPORTED_TYPE.include?(type) scanner.skip(/\s*/) blob = scanner.rest.unpack("m*").first keys << Net::SSH::Buffer.new(blob).read_key end end keys end |
#known_host_hash?(hostlist, entries) ⇒ Boolean
Indicates whether one of the entries matches an hostname that has been stored as a HMAC-SHA1 hash in the known hosts.
157 158 159 160 161 162 163 164 165 166 167 168 |
# File 'lib/net/ssh/known_hosts.rb', line 157 def known_host_hash?(hostlist, entries) if hostlist.size == 1 && hostlist.first =~ /\A\|1(\|.+){2}\z/ chunks = hostlist.first.split(/\|/) salt = Base64.decode64(chunks[2]) digest = OpenSSL::Digest.new('sha1') entries.each do |entry| hmac = OpenSSL::HMAC.digest(digest, salt, entry) return true if Base64.encode64(hmac).chomp == chunks[3] end end false end |